@@ -8,17 +8,13 @@
 
 use sysinfo::*;
 
-use gtk::gdk_pixbuf::Pixbuf;
-use gtk::gio::prelude::{ActionExt, ActionMapExt, ApplicationExt, ApplicationExtManual};
-use gtk::gio::MemoryInputStream;
-use gtk::glib::{Bytes, Cast, IsA, ToVariant};
-use gtk::prelude::{
-    AboutDialogExt, BoxExt, ButtonBoxExt, ButtonExt, ContainerExt, DialogExt, EntryExt,
-    GtkApplicationExt, GtkListStoreExt, GtkListStoreExtManual, GtkWindowExt, NotebookExtManual,
-    SearchBarExt, TreeModelExt, TreeSortableExtManual, TreeViewExt, WidgetExt, WidgetExtManual,
-};
-use gtk::{gdk, gio, glib};
-use gtk::{AboutDialog, Dialog, EditableSignals, Entry, Inhibit, MessageDialog};
+use gdk::Texture;
+use gdk_pixbuf::Pixbuf;
+use gio::MemoryInputStream;
+use glib::Bytes;
+use gtk::prelude::*;
+use gtk::{gdk, gdk_pixbuf, gio, glib};
+use gtk::{AboutDialog, Dialog, Entry, MessageDialog};
 
 use std::cell::RefCell;
 use std::collections::{HashMap, HashSet};
@@ -46,7 +42,6 @@ mod utils;
 use display_network::Network;
 use display_procs::{create_and_fill_model, Procs};
 use display_sysinfo::DisplaySysInfo;
-use notebook::NoteBook;
 use settings::Settings;
 use utils::format_number;
 
@@ -58,8 +53,8 @@ fn update_window(list: &gtk::ListStore, entries: &HashMap<Pid, sysinfo::Process>
     if let Some(iter) = list.iter_first() {
         let mut valid = true;
         while valid {
-            let pid = match list.value(&iter, 0).get::<u32>() {
-                Ok(pid) => pid as Pid,
+            let pid = match list.get_value(&iter, 0).get::<u32>() {
+                Ok(pid) => Pid::from_u32(pid),
                 _ => {
                     valid = list.iter_next(&iter);
                     continue;
@@ -68,7 +63,7 @@ fn update_window(list: &gtk::ListStore, entries: &HashMap<Pid, sysinfo::Process>
             if let Some(p) = entries.get(&(pid)) {
                 let disk_usage = p.disk_usage();
                 let disk_usage = disk_usage.written_bytes + disk_usage.read_bytes;
-                let memory = p.memory() * 1_000;
+                let memory = p.memory();
                 list.set(
                     &iter,
                     &[
@@ -103,7 +98,7 @@ fn update_window(list: &gtk::ListStore, entries: &HashMap<Pid, sysinfo::Process>
                 pro.cmd(),
                 pro.name(),
                 pro.cpu_usage(),
-                pro.memory() * 1_000,
+                pro.memory(),
             );
         }
     }
@@ -186,7 +181,7 @@ fn run_command<T: IsA<gtk::Window>>(input: &Entry, window: &T, d: &Dialog) {
         gtk::DialogFlags::DESTROY_WITH_PARENT,
         gtk::MessageType::Info,
         gtk::ButtonsType::Ok,
-        &x,
+        x,
     );
     m.set_modal(true);
     m.connect_response(|dialog, response| {
@@ -197,7 +192,7 @@ fn run_command<T: IsA<gtk::Window>>(input: &Entry, window: &T, d: &Dialog) {
             dialog.close();
         }
     });
-    m.show_all();
+    m.show();
 }
 
 fn create_new_proc_diag(
@@ -222,6 +217,7 @@ fn create_new_proc_diag(
     }
 }
 
+#[derive(Clone)]
 pub struct RequiredForSettings {
     process_refresh_timeout: Arc<Mutex<u32>>,
     network_refresh_timeout: Arc<Mutex<u32>>,
@@ -233,9 +229,8 @@ pub struct RequiredForSettings {
     network_tab: Rc<RefCell<Network>>,
 }
 
-fn setup_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
+fn setup_timeout(rfs: &RequiredForSettings) {
     let (ready_tx, ready_rx) = glib::MainContext::channel(glib::PRIORITY_DEFAULT);
-    let rfs = rfs.borrow();
 
     let sys = &rfs.sys;
     let process_dialogs = &rfs.process_dialogs;
@@ -243,7 +238,7 @@ fn setup_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
     let process_refresh_timeout = &rfs.process_refresh_timeout;
 
     thread::spawn(
-        glib::clone!(@weak sys, @strong ready_tx, @weak process_refresh_timeout => move || {
+        glib::clone!(@weak sys, @weak process_refresh_timeout => move || {
             loop {
                 let sleep_dur = Duration::from_millis(
                     *process_refresh_timeout.lock().expect("failed to lock process refresh mutex") as _);
@@ -260,7 +255,6 @@ fn setup_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
         let sorted = TreeSortableExtManual::sort_column_id(&list_store);
         list_store.set_unsorted();
 
-        let mut to_remove = 0;
         let mut dialogs = process_dialogs.borrow_mut();
 
         if let Ok(sys) = sys.lock() {
@@ -278,30 +272,24 @@ fn setup_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
                 } else {
                     dialog.set_dead();
                 }
-                if dialog.need_remove() {
-                    to_remove += 1;
-                }
             }
         } else {
             panic!("failed to lock sys to refresh UI");
         }
-        if to_remove > 0 {
-            dialogs.retain(|x| !x.need_remove());
-        }
+        dialogs.retain(|x| !x.need_remove());
         glib::Continue(true)
     }));
 }
 
-fn setup_network_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
+fn setup_network_timeout(rfs: &RequiredForSettings) {
     let (ready_tx, ready_rx) = glib::MainContext::channel(glib::PRIORITY_DEFAULT);
-    let rfs = rfs.borrow();
 
     let network_refresh_timeout = &rfs.network_refresh_timeout;
     let network_tab = &rfs.network_tab;
     let sys = &rfs.sys;
 
     thread::spawn(
-        glib::clone!(@weak sys, @strong ready_tx, @weak network_refresh_timeout => move || {
+        glib::clone!(@weak sys, @weak network_refresh_timeout => move || {
             loop {
                 let sleep_dur = Duration::from_millis(
                     *network_refresh_timeout.lock().expect("failed to lock networks refresh mutex") as _);
@@ -314,22 +302,21 @@ fn setup_network_timeout(rfs: &Rc<RefCell<RequiredForSettings>>) {
 
     ready_rx.attach(None,
         glib::clone!(@weak sys, @weak network_tab => @default-panic, move |_: bool| {
-            network_tab.borrow_mut().update_networks(&*sys.lock().expect("failed to lock to update networks"));
+            network_tab.borrow_mut().update_networks(&sys.lock().expect("failed to lock to update networks"));
             glib::Continue(true)
         })
     );
 }
 
-fn setup_system_timeout(rfs: &Rc<RefCell<RequiredForSettings>>, settings: &Rc<RefCell<Settings>>) {
+fn setup_system_timeout(rfs: &RequiredForSettings, settings: &Rc<RefCell<Settings>>) {
     let (ready_tx, ready_rx) = glib::MainContext::channel(glib::PRIORITY_DEFAULT);
-    let rfs = rfs.borrow();
 
     let system_refresh_timeout = &rfs.system_refresh_timeout;
     let sys = &rfs.sys;
     let display_tab = &rfs.display_tab;
 
     thread::spawn(
-        glib::clone!(@weak sys, @strong ready_tx, @weak system_refresh_timeout => move || {
+        glib::clone!(@weak sys, @weak system_refresh_timeout => move || {
             loop {
                 let sleep_dur = Duration::from_millis(
                     *system_refresh_timeout.lock().expect("failed to lock system refresh mutex") as _);
@@ -347,56 +334,66 @@ fn setup_system_timeout(rfs: &Rc<RefCell<RequiredForSettings>>, settings: &Rc<Re
             let sys = sys.lock().expect("failed to lock to update system");
             let display_fahrenheit = settings.borrow().display_fahrenheit;
 
-            info.update_system_info(&*sys, display_fahrenheit);
-            info.update_system_info_display(&*sys);
+            info.update_system_info(&sys, display_fahrenheit);
+            info.update_system_info_display(&sys);
             glib::Continue(true)
         }),
     );
 }
 
-fn build_ui(application: &gtk::Application) {
-    let settings = Settings::load();
+fn create_header_bar(stack: &gtk::Stack) -> (gtk::HeaderBar, gtk::Button) {
+    let header_buttons = gtk::StackSwitcher::new();
+    header_buttons.set_stack(Some(stack));
 
-    let menu = gio::Menu::new();
     let menu_bar = gio::Menu::new();
-    let more_menu = gio::Menu::new();
+
+    menu_bar.append(Some("Launch new executable"), Some("app.new-task"));
+
     let settings_menu = gio::Menu::new();
-
-    menu.append(Some("Launch new executable"), Some("app.new-task"));
-    menu.append(Some("Quit"), Some("app.quit"));
-    let quit = gio::SimpleAction::new("quit", None);
-    quit.connect_activate(glib::clone!(@weak application => move |_,_| {
-        application.quit();
-    }));
-    application.set_accels_for_action("app.quit", &["<Primary>Q"]);
-
     settings_menu.append(Some("Display temperature in °F"), Some("app.temperature"));
     settings_menu.append(Some("Display graphs"), Some("app.graphs"));
     settings_menu.append(Some("More settings..."), Some("app.settings"));
-    menu_bar.append_submenu(Some("_Settings"), &settings_menu);
+    menu_bar.append_section(None, &settings_menu);
 
+    let more_menu = gio::Menu::new();
     more_menu.append(Some("About"), Some("app.about"));
-    menu_bar.append_submenu(Some("?"), &more_menu);
+    menu_bar.append_section(None, &more_menu);
 
-    application.set_app_menu(Some(&menu));
-    application.set_menubar(Some(&menu_bar));
+    let menu = gio::Menu::new();
+    menu.append(Some("Quit"), Some("app.quit"));
+    menu_bar.append_section(None, &menu);
 
+    let menu_button = gtk::MenuButton::builder()
+        .icon_name("open-menu-symbolic")
+        .menu_model(&menu_bar)
+        .build();
+    menu_button.add_css_class("titlebar-button");
+
+    let search_filter_button = gtk::Button::from_icon_name("edit-find-symbolic");
+    search_filter_button.add_css_class("titlebar-button");
+
+    let header_bar = gtk::HeaderBar::new();
+
+    header_bar.pack_end(&menu_button);
+    header_bar.pack_end(&search_filter_button);
+    header_bar.set_title_widget(Some(&header_buttons));
+
+    (header_bar, search_filter_button)
+}
+
+fn build_ui(application: &gtk::Application) {
     let window = gtk::ApplicationWindow::new(application);
+    let stack = gtk::Stack::new();
+
+    let (header_bar, search_filter_button) = create_header_bar(&stack);
+    window.set_titlebar(Some(&header_bar));
 
     let mut sys =
         sysinfo::System::new_with_specifics(RefreshKind::everything().without_users_list());
-    let mut note = NoteBook::new();
-    let procs = Procs::new(sys.processes(), &mut note, &window);
+    let procs = Procs::new(sys.processes(), &stack);
     let current_pid = Rc::clone(&procs.current_pid);
     let info_button = procs.info_button.clone();
 
-    window.set_title("Process viewer");
-    window.set_position(gtk::WindowPosition::Center);
-    // To silence the annying warning:
-    // "(.:2257): Gtk-WARNING **: Allocating size to GtkWindow 0x7f8a31038290 without
-    // calling gtk_widget_get_preferred_width/height(). How does the code know the size to
-    // allocate?"
-    window.preferred_width();
     window.set_default_size(630, 700);
 
     sys.refresh_all();
@@ -410,26 +407,22 @@ fn build_ui(application: &gtk::Application) {
             }
         }));
 
-    let display_tab = DisplaySysInfo::new(&sys, &mut note, &settings);
+    let settings = Settings::load();
+    let display_tab = DisplaySysInfo::new(&sys, &stack, &settings);
 
     let settings = Rc::new(RefCell::new(settings));
-    let network_tab = Rc::new(RefCell::new(Network::new(&mut note, &window, &sys)));
-    display_disk::create_disk_info(&sys, &mut note);
-
-    let v_box = gtk::Box::new(gtk::Orientation::Vertical, 0);
+    let network_tab = Rc::new(RefCell::new(Network::new(&stack, &sys)));
+    display_disk::create_disk_info(&sys, &stack);
 
     let display_tab = Rc::new(RefCell::new(display_tab));
 
-    // I think it's now useless to have this one...
-    v_box.pack_start(&note.notebook, true, true, 0);
-
-    window.add(&v_box);
+    window.set_child(Some(&stack));
 
     let process_dialogs: Rc<RefCell<Vec<process_dialog::ProcDialog>>> =
         Rc::new(RefCell::new(Vec::new()));
     let list_store = procs.list_store.clone();
 
-    let rfs = Rc::new(RefCell::new(RequiredForSettings {
+    let rfs = RequiredForSettings {
         process_refresh_timeout: Arc::new(Mutex::new(settings.borrow().refresh_processes_rate)),
         network_refresh_timeout: Arc::new(Mutex::new(settings.borrow().refresh_network_rate)),
         system_refresh_timeout: Arc::new(Mutex::new(settings.borrow().refresh_system_rate)),
@@ -438,24 +431,27 @@ fn build_ui(application: &gtk::Application) {
         list_store,
         display_tab,
         network_tab: network_tab.clone(),
-    }));
+    };
 
     setup_timeout(&rfs);
     setup_network_timeout(&rfs);
     setup_system_timeout(&rfs, &settings);
 
     let settings_action = gio::SimpleAction::new("settings", None);
-    settings_action.connect_activate(glib::clone!(@weak settings, @weak rfs => move |_, _| {
+    settings_action.connect_activate(glib::clone!(@weak settings, @strong rfs => move |_, _| {
         settings::show_settings_dialog(&settings, &rfs);
     }));
 
     info_button.connect_clicked(
         glib::clone!(@weak current_pid, @weak process_dialogs, @weak sys => move |_| {
-                if let Some(pid) = current_pid.get() {
-                    create_new_proc_diag(&process_dialogs, pid, &*sys.lock().expect("failed to lock to create new proc dialog"));
-                }
+            if let Some(pid) = current_pid.get() {
+                create_new_proc_diag(
+                    &process_dialogs,
+                    pid,
+                    &sys.lock().expect("failed to lock to create new proc dialog"),
+                );
             }
-        ),
+        }),
     );
 
     procs
@@ -463,38 +459,41 @@ fn build_ui(application: &gtk::Application) {
         .connect_row_activated(glib::clone!(@weak sys => move |tree_view, path, _| {
                 let model = tree_view.model().expect("couldn't get model");
                 let iter = model.iter(path).expect("couldn't get iter");
-                let pid = model.value(&iter, 0)
+                let pid = model.get_value(&iter, 0)
                                .get::<u32>()
                                .expect("Model::get failed");
-                create_new_proc_diag(&process_dialogs, pid as Pid, &*sys.lock().expect("failed to lock to create new proc dialog (from tree)"));
+                create_new_proc_diag(
+                    &process_dialogs,
+                    Pid::from_u32(pid),
+                    &sys.lock().expect("failed to lock to create new proc dialog (from tree)"),
+                );
             }
         ));
 
     let about = gio::SimpleAction::new("about", None);
     about.connect_activate(glib::clone!(@weak window => move |_, _| {
-        let p = AboutDialog::new();
-        p.set_authors(&["Guillaume Gomez"]);
-        p.set_website_label(Some("my website"));
-        p.set_website(Some("https://guillaume-gomez.fr/"));
-        p.set_comments(Some("A process viewer GUI written with gtk-rs"));
-        p.set_copyright(Some("Licensed under MIT"));
-        p.set_transient_for(Some(&window));
-        p.set_program_name("process-viewer");
-        let memory_stream = MemoryInputStream::from_bytes(
-            &Bytes::from_static(include_bytes!(
-                    concat!(env!("CARGO_MANIFEST_DIR"), "/assets/eye.png"))));
-        let logo = Pixbuf::from_stream(&memory_stream, None::<&gio::Cancellable>);
-        if let Ok(logo) = logo {
-            p.set_logo(Some(&logo));
-        }
-        p.set_modal(true);
-        p.connect_response(|dialog, response| {
-            if response == gtk::ResponseType::DeleteEvent ||
-               response == gtk::ResponseType::Close {
-                dialog.close();
-            }
-        });
-        p.show_all();
+        let p = AboutDialog::builder()
+            .authors(vec!["Guillaume Gomez".to_owned()])
+            .website_label("my website")
+            .website("https://guillaume-gomez.fr/")
+            .comments("A process viewer GUI written with gtk-rs")
+            .copyright("Licensed under MIT")
+            .program_name("process-viewer")
+            .transient_for(&window)
+            .modal(true);
+
+        let bytes = Bytes::from_static(include_bytes!(
+            concat!(env!("CARGO_MANIFEST_DIR"), "/assets/eye.png")));
+        let memory_stream = MemoryInputStream::from_bytes(&bytes);
+        let pixbuf = Pixbuf::from_stream(&memory_stream, gio::Cancellable::NONE);
+        let p = if let Ok(pixbuf) = pixbuf {
+            let logo = Texture::for_pixbuf(&pixbuf);
+            p.logo(&logo)
+        } else {
+            p
+        };
+
+        p.build().show();
     }));
 
     let new_task = gio::SimpleAction::new("new-task", None);
@@ -502,31 +501,23 @@ fn build_ui(application: &gtk::Application) {
         let dialog = gtk::Dialog::with_buttons(
             Some("Launch new executable"),
             Some(&window),
-            gtk::DialogFlags::USE_HEADER_BAR,
+            gtk::DialogFlags::MODAL,
             &[("Run", gtk::ResponseType::Other(0)), ("Cancel", gtk::ResponseType::Close)],
         );
-        let input = Entry::new();
+        let input = Entry::builder()
+            .css_classes(vec!["button-with-margin".to_owned()])
+            .vexpand(false)
+            .hexpand(true)
+            .build();
 
         // To set "run" button disabled by default.
         dialog.set_response_sensitive(gtk::ResponseType::Other(0), false);
-        // To make "run" and "cancel" button take all spaces.
-        if let Some(run) = dialog.widget_for_response(gtk::ResponseType::Other(0)) {
-            if let Some(parent) = run.parent() {
-                match parent.downcast::<gtk::ButtonBox>() {
-                    Ok(parent) => parent.set_layout_style(gtk::ButtonBoxStyle::Expand),
-                    Err(e) => {
-                        eprintln!(
-                            "<Process_Viewer::build_ui> Failed to set layout style for new task \
-                             button box: {}", e)
-                    }
-                }
-            }
-        }
+
         input.connect_changed(glib::clone!(@weak dialog => move |input| {
             if !input.text().is_empty() {
-                    dialog.set_response_sensitive(gtk::ResponseType::Other(0), true);
-               }
-               else { dialog.set_response_sensitive(gtk::ResponseType::Other(0), false);
+                dialog.set_response_sensitive(gtk::ResponseType::Other(0), true);
+            } else {
+                dialog.set_response_sensitive(gtk::ResponseType::Other(0), false);
             }
         }));
         input.connect_activate(glib::clone!(@weak window, @weak dialog => move |input| {
@@ -544,25 +535,19 @@ fn build_ui(application: &gtk::Application) {
             }
         }));
 
-        dialog.content_area().add(&input);
-        // To silence the annying warning:
-        // "(.:2257): Gtk-WARNING **: Allocating size to GtkWindow 0x7f8a31038290 without
-        // calling gtk_widget_get_preferred_width/height(). How does the code know the size to
-        // allocate?"
-        dialog.preferred_width();
+        dialog.content_area().append(&input);
         dialog.set_size_request(400, 70);
-        dialog.show_all();
+        dialog.show();
     }));
 
     let graphs = gio::SimpleAction::new_stateful(
         "graphs",
         None,
-        &settings.borrow().display_graph.to_variant(),
+        settings.borrow().display_graph.to_variant(),
     );
-    graphs.connect_activate(glib::clone!(@weak settings, @weak rfs => move |g, _| {
+    graphs.connect_activate(glib::clone!(@weak settings => move |g, _| {
         let mut is_active = false;
         if let Some(g) = g.state() {
-            let rfs = rfs.borrow();
             is_active = g.get().expect("couldn't get bool");
             rfs.display_tab.borrow().set_checkboxes_state(!is_active);
         }
@@ -577,7 +562,7 @@ fn build_ui(application: &gtk::Application) {
     let temperature = gio::SimpleAction::new_stateful(
         "temperature",
         None,
-        &settings.borrow().display_fahrenheit.to_variant(),
+        settings.borrow().display_fahrenheit.to_variant(),
     );
     temperature.connect_activate(move |g, _| {
         let mut is_active = false;
@@ -591,6 +576,17 @@ fn build_ui(application: &gtk::Application) {
         settings.borrow_mut().display_fahrenheit = !is_active;
         settings.borrow().save();
     });
+    let quit = gio::SimpleAction::new("quit", None);
+    quit.connect_activate(glib::clone!(@weak application => move |_,_| {
+        application.quit();
+    }));
+    application.set_accels_for_action("app.quit", &["<Primary>Q"]);
+    let finder = gio::SimpleAction::new("finder", None);
+    // Little hack to correctly handle `ctrl+F` shortcut.
+    finder.connect_activate(glib::clone!(@weak search_filter_button => move |_,_| {
+        search_filter_button.emit_clicked();
+    }));
+    application.set_accels_for_action("app.finder", &["<Primary>F"]);
 
     application.add_action(&about);
     application.add_action(&graphs);
@@ -598,75 +594,102 @@ fn build_ui(application: &gtk::Application) {
     application.add_action(&settings_action);
     application.add_action(&new_task);
     application.add_action(&quit);
+    application.add_action(&finder);
 
     window.set_widget_name(utils::MAIN_WINDOW_NAME);
 
-    window.add_events(gdk::EventMask::STRUCTURE_MASK);
-    // TODO: ugly way to resize drawing area, I should find a better way
-    window.connect_configure_event(move |w, _| {
-        // To silence the annoying warning:
-        // "(.:2257): Gtk-WARNING **: Allocating size to GtkWindow 0x7f8a31038290 without
-        // calling gtk_widget_get_preferred_width/height(). How does the code know the size to
-        // allocate?"
-        w.preferred_width();
-        let w = w.size().0 - 130;
-        let rfs = rfs.borrow();
-        rfs.display_tab.borrow().set_size_request(w, 200);
-        false
-    });
-
-    application.connect_activate(glib::clone!(@weak procs.filter_entry as filter_entry, @weak network_tab, @weak window => move |_| {
-        window.show_all();
-        filter_entry.hide();
-        network_tab.borrow().filter_entry.hide();
+    application.connect_activate(glib::clone!(@weak window => move |_| {
         window.present();
     }));
 
-    window.connect_key_press_event(
-        glib::clone!(@weak note.notebook as notebook => @default-return Inhibit(false), move |win, key| {
-            let current_page = notebook.current_page();
-            if current_page == Some(0) || current_page == Some(2) {
-                // the process list
-                if key.keyval() == gdk::keys::constants::Escape {
-                    if current_page == Some(0) {
-                        procs.hide_filter();
-                    } else {
-                        network_tab.borrow().hide_filter();
-                    }
-                } else if current_page == Some(0) {
-                    let ret = procs.search_bar.handle_event(key);
-                    if !procs.filter_entry.text().is_empty() {
-                            procs.filter_entry.show_all();
-                            if win.focus()
-                                != Some(procs.filter_entry.clone().upcast::<gtk::Widget>())
-                            {
-                                win.set_focus(Some(&procs.filter_entry));
-                            }
-                    }
-                    return Inhibit(ret);
-                } else {
-                    let network = network_tab.borrow();
-                    let ret = network.search_bar.handle_event(key);
-                    if !network.filter_entry.text().is_empty() {
-                            network.filter_entry.show_all();
-                            if win.focus()
-                                != Some(network.filter_entry.clone().upcast::<gtk::Widget>())
-                            {
-                                win.set_focus(Some(&network.filter_entry));
-                            }
-                    }
-                    return Inhibit(ret);
+    procs.search_bar.set_key_capture_widget(Some(&window));
+
+    fn revert_display(search_bar: &gtk::SearchBar) {
+        if search_bar.is_search_mode() {
+            search_bar.set_search_mode(false);
+        } else {
+            search_bar.set_search_mode(true);
+        }
+    }
+
+    search_filter_button.connect_clicked(glib::clone!(
+        @strong stack,
+        @weak procs.search_bar as procs_search_bar,
+        @weak network_tab,
+        => move |_| {
+        if let Some(name) = stack.visible_child_name() {
+            match name.as_str() {
+                "Processes" => revert_display(&procs_search_bar),
+                "Networks" => revert_display(&network_tab.borrow().search_bar),
+                _ => {}
+            };
+        }
+    }));
+
+    stack.connect_visible_child_notify(move |s| {
+        if let Some(name) = s.visible_child_name() {
+            match name.as_str() {
+                "Processes" => {
+                    procs.search_bar.set_key_capture_widget(Some(&window));
+                    network_tab
+                        .borrow()
+                        .search_bar
+                        .set_key_capture_widget(gtk::Widget::NONE);
+                    search_filter_button.set_sensitive(true);
+                    return;
                 }
+                "Networks" => {
+                    network_tab
+                        .borrow()
+                        .search_bar
+                        .set_key_capture_widget(Some(&window));
+                    procs.search_bar.set_key_capture_widget(gtk::Widget::NONE);
+                    search_filter_button.set_sensitive(true);
+                    return;
+                }
+                _ => {}
             }
-            Inhibit(false)
-        }),
-    );
+        }
+        search_filter_button.set_sensitive(false);
+        procs.search_bar.set_key_capture_widget(gtk::Widget::NONE);
+        network_tab
+            .borrow()
+            .search_bar
+            .set_key_capture_widget(gtk::Widget::NONE);
+    });
 }
 
 fn main() {
     let application = gtk::Application::new(Some(APPLICATION_NAME), gio::ApplicationFlags::empty());
 
     application.connect_startup(move |app| {
+        let provider = gtk::CssProvider::new();
+        // Style needed for graph.
+        provider.load_from_data(
+            r#"
+graph_widget {
+    color: @theme_fg_color;
+}
+
+.titlebar-button {
+    background-color: @theme_bg_color;
+    border-radius: 4px;
+}
+.titlebar-button:hover {
+    background-color: shade(@theme_bg_color, 1.50);
+}
+
+.button-with-margin {
+    margin: 6px;
+}
+"#,
+        );
+        gtk::StyleContext::add_provider_for_display(
+            &gdk::Display::default().expect("Could not connect to a display."),
+            &provider,
+            gtk::STYLE_PROVIDER_PRIORITY_APPLICATION,
+        );
+
         build_ui(app);
     });
 
